#!/usr/bin/env python3

import click
from pydamage.main import analyze
from pydamage import __version__

@click.command()
@click.version_option(__version__)
@click.argument('bam', type=click.Path(exists=True))
@click.option('-w',
              '--wlen',
              default=20,
              type=int,
              show_default=True,
              help='Window length for damage modeling')
@click.option('-p',
              '--process',
              default=2,
              type=int,
              show_default=True,
              help='Number of processes')
@click.option('-m',
              '--mini',
              default=1000,
              type=int,
              show_default=True,
              help='Minimum reads aligned to consider reference')
@click.option('-c',
              '--cov',
              default=8,
              type=float,
              show_default=True,
              help='Minimum coverage to consider reference')
@click.option('-s',
              '--show_al',
              is_flag=True,
              help='Show alignments representations')
@click.option('-pl',
              '--plot',
              is_flag=True,
              help='Make the damage plots')
@click.option('--verbose', is_flag=True, help='Verbose mode')
@click.option('-o',
              '--outdir',
              type=click.Path(writable=True, dir_okay=True),
              default="pydamage_results",
              show_default=True,
              help="Output directory")
@click.option('--force', 
              is_flag=True, 
              help='Force overwriting of results directory')
    
def cli(no_args_is_help=True, **kwargs):
    """\b
    PyDamage: Damage parameter estimation for ancient DNA
    Author: Maxime Borry
    Contact: <borry[at]shh.mpg.de>
    Homepage & Documentation: github.com/maxibor/pydamage

    BAM: path to BAM/SAM/CRAM alignment file
    """
    analyze(**kwargs)


if __name__ == "__main__":
    cli()



#!/usr/bin/env python3
import pysam
from pydamage import utils
import multiprocessing
from functools import partial
from pydamage import damage
from pydamage.plot import damageplot
import pandas as pd
import sys
from tqdm import tqdm
import warnings
from pydamage import __version__

def analyze(bam, wlen=30, show_al=False, mini=2000, cov=0.5, process=1, outdir="", plot = False, verbose=False, force=False):
    """Runs the pydamage analysis

    Args:
        bam(str): Path to alignment (sam/bam/cram) file
        wlen(int): window length
        show_al(bool): print alignments representations
        mini(int):  Minimum numbers of reads aligned  to consider contigs
        cov(float): Minimum coverage to consider contig
        process(int):  Number of  processes for parellel computing
        outdir(str): Path to output directory
        verbose(bool): verbose mode
        force(bool): force overwriting of results directory
    Returns:
        pd.DataFrame: pandas DataFrame containg pydamage results

    """
    if verbose:
        print(f"Pydamage version {__version__}\n")
    utils.makedir(outdir, force=force)

    if not verbose:
        warnings.filterwarnings("ignore")

    mode = utils.check_extension(bam)
    alf = pysam.AlignmentFile(bam, mode)

    if not alf.has_index():
        print(f"BAM file {bam} has no index. Sort BAM file and provide index "
               "before running pydamage.")
        sys.exit(1)

    refs = list(alf.references)

    if len(refs) == 0:
        print(f"No aligned sequences in {bam}")
        return([])

    proc = min(len(refs), process)
    
    ###########################
    # Simple loop for debugging
    ###########################
    # filt_res = []
    # for ref in refs:
    #     res = damage.test_damage(bam=bam, ref=ref, wlen=wlen,
    #                          min_al=mini,  min_cov=cov, show_al=show_al,
    #                          mode=mode, process=process, verbose=verbose)
    #     if res:
    #         filt_res.append(res)
    ###########################
    ###########################
    

    test_damage_partial = partial(damage.test_damage, bam=bam, wlen=wlen,
                              min_al=mini, min_cov=cov, show_al=show_al,
                              mode=mode, process=process, verbose=verbose)
    print("Estimating and testing Damage")
    with multiprocessing.Pool(proc) as p:
        res = tqdm(p.imap(test_damage_partial, refs), total = len(refs))
        filt_res = [i for i in res if i]


    print(f"{len(filt_res)} contigs were successfully analyzed by Pydamage")

    if plot and len(filt_res) > 0:
        print("\nGenerating Pydamage plots")
        plotdir = f"{outdir}/plots"
        utils.makedir(plotdir, confirm=False)

        plot_partial = partial(damageplot, wlen=wlen, outdir=plotdir)
        with multiprocessing.Pool(proc) as p:
            list(tqdm(p.imap(plot_partial, filt_res),total=len(filt_res)))
            # p.close()
            # p.join()
        # for ref in tqdm(filt_res):
        #     dam_plot = damageplot(damage_dict=ref, wlen=wlen, outdir=plotdir)
    df = utils.pandas_processing(res_dict=filt_res, outdir=outdir)
    return(df)
    



#!/usr/bin/env python3

import numpy as np
from scipy.stats import norm
from pydamage.optim import optim
import collections
from pydamage.utils import sort_dict_by_keys

def vuong_closeness(ref, model_A, model_B, ct_data, ga_data, all_bases, wlen, verbose):
    """Performs model fitting and runs Vuong's closeness test

    Args:
        ref (str): name of referene in alignment file
        model_A (pydamage.models): Pydamage H1 model
        model_B (pydamage.models): Pydamage H0 model
        ct_data (list of int): List of positions where CtoT transitions were observerd
        ga_data (list of int): List of positions where GtoA transitions were observerd
        all_bases (list of int): List of positions where a base is aligned
        wlen (int): window length
        verbose (bool): verbose mode
    """
    all_bases_pos, all_bases_counts = np.unique(np.sort(all_bases), return_counts=True)
    c2t_pos, c2t_counts = np.unique(np.sort(ct_data), return_counts=True)
    g2a_pos, g2a_counts = np.unique(np.sort(ga_data), return_counts=True)
    c2t = dict(zip(c2t_pos, c2t_counts))
    g2a = dict(zip(g2a_pos, g2a_counts))

    # Adding zeros at positions where no damage is observed
    for i in all_bases_pos:
        if all_bases_counts[i] > 0:
            if i not in c2t:
                c2t[i] = 0
            if i not in g2a:
                g2a[i] = 0
    c2t = sort_dict_by_keys(c2t)
    g2a = sort_dict_by_keys(g2a)

    xdata = np.array(list(c2t.keys()))
    counts = np.array(list(c2t.values()))

    ydata = list(counts/counts.sum())
    qlen = len(ydata)
    ydata_counts = {i:c for i,c in enumerate(ydata)}
    ctot_out = {f"CtoT-{k}":v for k,v in enumerate(ydata)}
    
    g2a_counts = np.array(list(g2a.values()))
    y_ga = list(g2a_counts/g2a_counts.sum())
    y_g2a_counts = {i:c for i,c in enumerate(y_ga)}
    gtoa_out = {f"GtoA-{k}":v for k,v in enumerate(y_ga)}

   
    # for i in all_bases_pos:
    #     print(f"{ref},{i},{all_bases_counts[i]}")
    for i in range(qlen):
        if i not in ydata_counts :
            ydata_counts[i] = np.nan
        if f"CtoT-{i}" not in ctot_out:
            ctot_out[f"CtoT-{i}"] = np.nan
        if f"GtoA-{i}" not in gtoa_out:
            gtoa_out[f"GtoA-{i}"] = np.nan


    xdata = xdata[:wlen]
    ydata = ydata[:wlen]
    ct_data = np.array(ct_data)
    ct_data = ct_data[ct_data<wlen]

    res = {}
    optim_A, stdev_A = optim(function=model_A.pmf,
                    parameters=model_A.kwds,
                    xdata=xdata,
                    ydata=ydata,
                    bounds=model_A.bounds)
    optim_B, stdev_B = optim(function=model_B.pmf,
                    parameters=model_B.kwds,
                    xdata=xdata,
                    ydata=ydata,
                    bounds=model_B.bounds)

    LA = model_A.log_pmf(x=ct_data, **optim_A)
    LB = model_B.log_pmf(x=ct_data, **optim_B)
    pdiff = len(model_A.kwds) - len(model_B.kwds)
    LR = LA.sum() - LB.sum() - (pdiff/2)*np.log(len(ct_data))
    omega = np.std(LA-LB)
    Z = LR/(np.sqrt(len(ct_data))*omega)
    pval = norm.cdf(Z)
    # if verbose:
    #     print(f"\nReference: {ref}")
    #     print(f"Vuong closeness test Z-score for {ref}: {round(Z, 4)}")
    #     print(f"Vuong closeness test p-value for {ref}: {round(pval, 4)}")
    #     print(f"Model A parameters for {ref}: {optim_A}")
    #     print(f"Model B parameters for {ref}: {optim_B}")
    res.update(ydata_counts)
    res.update(ctot_out)
    res.update(gtoa_out)
    res.update(optim_A)
    res.update(stdev_A)
    res.update(optim_B)
    res.update(stdev_B)
    res.update({'pvalue': pval})
    res.update({'base_cov': all_bases_counts})
    res.update({'model_params': list(optim_A.values()) + list(optim_B.values())+list(stdev_A.values())+list(stdev_B.values())})
    res['qlen'] = qlen
    return(res)



#!/usr/bin/env python3

import pysam
from pydamage.parse_damage import damage_al
from pydamage.vuong import vuong_closeness
from pydamage import models
import numpy as np

class al_to_damage():

    def __init__(self, reference, al_handle):
        """Constructor of the class

        Args:
            reference (string): a reference from the indexed bam file 
            al_handle(pysam.AlignmentFile)
        """
        self.alignments = al_handle.fetch(reference)

        # self.alignments = al_file

    # def __repr__(self):
    #     return(f"Reference {reference}")

    def get_damage(self, wlen, show_al):
        """Compute CtoT substitutions

        Args:
            wlen (int): window length
            show_al(bool): print alignments representations
        Returns:
            list: all_ct - positions of reads with CtoT transitions
            list: all_ga - positions of reads with GtoA transitions
        """

        all_ct = []
        all_ga = []
        all_bases = []
        for al in self.alignments:
            if al.is_reverse == False and al.is_unmapped == False:
                cigar = al.cigartuples
                ref = al.get_reference_sequence()
                quer = al.query_sequence

                all_damage = damage_al(reference=ref,
                                       query=quer,
                                       cigartuple=cigar,
                                       wlen=wlen,
                                       show_al=show_al)
                all_ct += all_damage['CT']
                all_ga += all_damage['GA']
                all_bases += all_damage['all']

        return(all_ct, all_ga, all_bases)


def avg_coverage(pysam_cov):
    """Computes average coverage of a reference

    Args:
        pysam_cov (np.array): Four dimensional array of coverage for each base
    Returns:
        float: mean coverage of reference
    """
    A = np.array(pysam_cov[0], dtype=int)
    C = np.array(pysam_cov[1], dtype=int)
    G = np.array(pysam_cov[2], dtype=int)
    T = np.array(pysam_cov[3], dtype=int)
    cov_all_bases = A + C + G + T
    cov = np.mean(cov_all_bases)
    return(cov)

def check_model_fit(model_dict, wlen, verbose):
    """Check if model fitting makes sense

    Args:
        model_dict (dict): Dictionary containing Vuong test results
        wlen (int): window length
        verbose (bool): Run in verbose mode
    Returns:
        bool or model_dict(dict): False if test fails, model_dict otherwise
    """
    # Check that no fitted parameters or stdev are infinite
    if np.isinf(np.array(model_dict['model_params'])).any():
        if verbose:
            print(f"Unreliable model fit for {model_dict['reference']}")
        return(False)

    # Check that all the first wlen bases are covered
    if np.any(np.array(model_dict['base_cov'][:wlen])==0):
        if verbose:
            print(f"Could not reliably fit a model to {model_dict['reference']} because of too few reads aligned")
        return(False)
    return(model_dict)

def test_damage(ref, bam, mode, wlen, show_al, min_al, min_cov, process, verbose):
    """Prepare data and run Vuong's test to test for damage

    Args:
        ref (str): name of referene in alignment file
        bam (str): bam file
        mode (str): opening mode of alignment file
        wlen (int): window length
        show_al (bool): Show alignment representations
        min_al (int): Minimum  number of aligned reads
        min_cov (float): Minimum coverage
        process (int): Number of process for parallelization
        verbose (bool): Run in verbose mode
    Returns:
        dict: Dictionary containing Vuong test results
    """
    al_handle = pysam.AlignmentFile(bam, mode=mode, threads=process)
    try:
        cov = avg_coverage(al_handle.count_coverage(contig=ref))
        nb_reads_aligned = al_handle.count(contig=ref)
        reflen = al_handle.get_reference_length(ref)
        
        if nb_reads_aligned >= min_al or cov >= min_cov:
            al = al_to_damage(reference=ref, al_handle=al_handle)
            ct_data, ga_data, all_bases = al.get_damage(wlen=wlen, show_al=show_al)
            if ct_data:
                model_A = models.unif_mod()
                model_B = models.geom_mod()
                test_res = vuong_closeness(ref=ref, 
                                            model_A=model_A, 
                                            model_B=model_B, 
                                            ct_data=ct_data, 
                                            ga_data=ga_data,
                                            all_bases=all_bases,
                                            wlen=wlen, 
                                            verbose=verbose)
                test_res['reference'] = ref
                test_res['nb_reads_aligned'] = nb_reads_aligned
                test_res['coverage'] = cov

                return(check_model_fit(test_res, wlen, verbose))
        else:
            if verbose:
                print(f"Did not attempt to fit a model to {ref} because of too few reads aligned")
                print(f"nb_reads_aligned: {nb_reads_aligned} - coverage: {cov} - reflen: {reflen}\n")
            pass
    except ValueError as e:
        if verbose:
            print(f"Model fitting for {ref} failed because of too few reads aligned")
            print(f"Model fitting error: {e}")
            print(f"nb_reads_aligned: {nb_reads_aligned} - coverage: {cov} - reflen: {reflen}\n")
        return(False)



#!/usr/bin/env python3

import numpy as np


class geom_mod():
    def __init__(self):
        self.kwds = ['geom_p', 'geom_pmin', 'geom_pmax']
        self.bounds = ((0.0, 0.0, 0.0), (0.99, 0.2, 0.99))

    def __repr__(self):
        return(
            f"""
            A modified geometric function of formula:
            y = ((((1-geom_p)**x)*geom_p) - xmin)/(xmax - xmin))*(geom_pmax - geom_pmin) + geom_pmin
            With parameters:
            - geom_p
            - geom_pmin
            - geom_pmax
            - xmin
            - xmax
            """
        )

    def pmf(self, x, geom_p, geom_pmin, geom_pmax):
        """Probability mass function of the discrete geometric function

        Args:
            x (numpy array) data
            geom_p (float): shape parameter
            geom_pmin (float): min y value
            geom_pmax (float): max y value
        Returns:
            np.array: PMF(x)
        """
        base_geom = ((1-geom_p)**x)*geom_p
        xmin = min(base_geom)
        xmax = max(base_geom)
        scaled_geom = ((base_geom - xmin)/(xmax - xmin)) * \
            (geom_pmax - geom_pmin) + geom_pmin
        return(scaled_geom)

    def log_pmf(self, x, geom_p, geom_pmin, geom_pmax):
        """Log Probability mass function of the discrete geometric function

        Args:
            x (numpy array) data
            geom_p (float): shape parameter
            geom_pmin (float): min y value
            geom_pmax (float): max y value
        Returns:
            np.array: LogPMF(x)
        """
        return(np.log(self.pmf(x=x, geom_p=geom_p, geom_pmin=geom_pmin, geom_pmax=geom_pmax)))


class unif_mod():
    def __init__(self):
        self.kwds = ('unif_pmin',)
        self.bounds = ((0.0,), (0.2,))

    def __repr__(self):
        return(
            f"""
            A modified uniform function of formula
            y = unif_pmin
            With parameters:
            - unif_pmin
            """
        )

    def pmf(self, x, unif_pmin):
        """Probability mass function of the discrete geometric function

        Args:
            x (numpy array) data
            unif_pmin (float): target y value
        Returns:
            np.array: PMF(x)
        """
        return(np.array([unif_pmin]*len(x)))

    def log_pmf(self, x, unif_pmin):
        """Log probability mass function of the discrete geometric function

        Args:
            x (numpy array) data
            geom_pmin (float): target y value
        Returns:
            np.array: LogPMF(x)
        """
        return(np.log(self.pmf(x=x, unif_pmin=unif_pmin)))



import numpy as np
import os
import sys
import shutil
from statsmodels.stats.multitest import multipletests
import pandas as pd



def check_extension(filename):
    """Check alignment file format to give correct open mode

    Args:
        filename (str): Path to alignment file

    Returns:
        str: opening mode

    Raises:
        Exception: Extension not supported
    """
    extension = filename.split(".")[-1]
    modes = {'bam': 'rb', 'sam': 'r', 'cram': 'rc'}
    try:
        return(modes[extension])
    except KeyError:
        raise Exception(f"{extension} file extension not supported")


def makedir(dirpath, confirm=True, force=False):
    """Make directory if user confirms overwritting

    Args:
        dirpath (str): Path to directory
        confirm (bool, optional): Ask the user to confirm. Defaults to True.
        force (bool, optional): Always create directory. Defaults to False.
    """
    if os.path.exists(dirpath):
        if confirm and force==False:
            print(f"Result directory, {dirpath}, already exists, it will be overwritten")
            if input('Do You Want To Continue? (y|n) ').lower() != 'y':
                sys.exit()
        shutil.rmtree(dirpath)
    
    os.makedirs(dirpath)

def pandas_processing(res_dict, outdir):
    """Performs Pandas processing of Pydamage results

    Args:
        res_dict (dict): Result dictionary of Vuong's closeness test
        outdir (str): Path to output directory
    """
    df = pd.DataFrame(res_dict)
    if len(res_dict) == 0:
        return(df)
    qvalues = pd.Series(multipletests(df['pvalue'].dropna(), method='fdr_bh')[1], index=df['pvalue'].dropna().index, name='qvalue')
    df = df.merge(qvalues, left_index=True, right_index=True, how='outer')
    df = df[['unif_pmin', 'unif_pmin_stdev', 
             'geom_p', 'geom_p_stdev',
             'geom_pmin', 'geom_pmin_stdev',
             'geom_pmax', 'geom_pmax_stdev',
             'pvalue', 
             'qvalue', 
             'reference',
             'nb_reads_aligned',
             'coverage']+
             [f"CtoT-{i}" for i in range(df['qlen'].max())]+
             [f"GtoA-{i}" for i in range(df['qlen'].max())]]
    df.sort_values(by=['qvalue'], inplace=True)
    df.set_index("reference", inplace=True)
    df.dropna(axis=1, how='all', inplace=True)

    df.to_csv(f"{outdir}/pydamage_results.csv")
    return(df)

def sort_dict_by_keys(adict):
    """Sort dictonary by keys

    Args:
        adict (dict): dictorary to sort
    Returns:
        dict: dictionary sorted on keys
    """
    res = {}
    for k in sorted(adict.keys()):
        res[k] = adict[k]
    return(res)


import matplotlib.pyplot as plt
from pydamage.models import geom_mod, unif_mod
from pydamage import utils
import numpy as np
from os import makedirs


def damageplot(damage_dict, wlen, outdir):
        """Draw pydamage plots
        
        Args:
            damage_dict(dict): pydamage result dictionary
            wlen(int): window length
            qlen(int): query length
            outdir(str): Pydamage result directory
        """        
        x = np.array(range(wlen))
        qlen = np.array(range(damage_dict['qlen']))
        y = np.array([damage_dict[i] for i in x])
        c2t = np.array([damage_dict[f"CtoT-{i}"] for i in qlen])
        g2a = np.array([damage_dict[f"GtoA-{i}"] for i in qlen])
        unif_pmin = damage_dict['unif_pmin']
        unif_pmin_stdev = damage_dict['unif_pmin_stdev']
        geom_p = damage_dict['geom_p']
        geom_pmin = damage_dict['geom_pmin']
        geom_pmin_stdev = damage_dict['geom_pmin_stdev']
        geom_pmax = damage_dict['geom_pmax']
        geom_pmax_stdev = damage_dict['geom_pmax_stdev']
        contig = damage_dict['reference']
        pvalue = damage_dict['pvalue']
        coverage = damage_dict['coverage']
        plotdir = outdir

        if pvalue < 0.001:
            rpval = "<0.001"
        else:
            rpval = f"={round(pvalue,3)}"


        unif = unif_mod()
        unif_pmin_low = max(unif.bounds[0][0], unif_pmin - 2*unif_pmin_stdev)
        unif_pmin_high = min(unif.bounds[1][0], unif_pmin + 2*unif_pmin_stdev)
        y_unif = unif.pmf(x, unif_pmin)
        y_unif_low = np.maximum(np.zeros(y_unif.shape[0]), unif.pmf(x, unif_pmin_low)) 
        y_unif_high = np.minimum(np.ones(y_unif.shape[0]), unif.pmf(x, unif_pmin_high))

        geom = geom_mod()
        geom_pmin_low = max(geom.bounds[0][1], geom_pmin - 2*geom_pmin_stdev)
        geom_pmin_high = min(geom.bounds[1][1], geom_pmin + 2*geom_pmin_stdev)
        geom_pmax_low = max(geom.bounds[0][2], geom_pmax - 2*geom_pmax_stdev)
        geom_pmax_high = min(geom.bounds[1][2], geom_pmax + 2*geom_pmax_stdev)

        y_geom = geom.pmf(x, geom_p, geom_pmin, geom_pmax)
        y_geom_low = np.maximum(np.zeros(y_geom.shape[0]), geom.pmf(x, geom_p, geom_pmin_low, geom_pmax_low))
        y_geom_high = np.minimum(np.ones(y_geom.shape[0]), geom.pmf(x, geom_p, geom_pmin_high, geom_pmax_high))

        fig = plt.figure(figsize=(12,8), dpi=100, facecolor='w', edgecolor='k')
        ax = fig.add_subplot(111)
        ax.xaxis.labelpad = 20
        ax.yaxis.labelpad = 20

        plt.plot(qlen, c2t,
                color='#bd0d45',
                alpha=0.1,
                label='C to T transitions')

        plt.plot(qlen, g2a,
                color='#236cf5',
                alpha=0.1,
                label='G to A transitions')

        plt.plot(x, y_unif, 
            linewidth=2.5, 
            color = 'DarkOliveGreen',
            alpha = 0.8,
            label = 'Uniform model')

        plt.fill_between(x, y_unif_low, y_unif_high,
            color='DarkOliveGreen',
            alpha=0.1,
            label = 'Uniform CI (2 sigma)')

        plt.plot(x, y_geom,
                linewidth=2.5, 
                color = '#D7880F',
                alpha = 0.8,
                label = 'Geometric model')

        plt.fill_between(x, y_geom_low, y_geom_high,
            color='#D7880F',
            alpha=0.1,
            label = 'Geometric CI (2 sigma)')

        plt.xlabel("Base from 5'", fontsize=20)
        plt.ylabel("Substitution frequency", fontsize=20)
        plt.suptitle(f"{contig}", fontsize=20, y=0.95)
        plt.title(f"coverage: {round(coverage,2)} | pvalue{rpval}",fontsize=12)
        plt.xticks(rotation=45, fontsize=8)
        ax.legend(fontsize=12)
        ax.set_xticks(qlen)
        plt.savefig(f"{plotdir}/{contig}.png", dpi=200)


#!/usr/bin/env python3


def damage_al(reference, query, cigartuple, wlen, show_al):
    """Compute CtoT mutations for a single alignment

    Args:
        reference (string): reference sequence
        query (string): query sequence
        cigartuple (tuple): cigar tuple (pysam)
        wlen (int): window length
        print_al (bool): print alignment
    Returns:
        dict : {'CT': [ CtoT pos], 
                'GA': [GtoA pos], 
                'A':[A pos],
                'T':[T pos],
                'G':[G pos],
                'C':[C pos],
                'N':[N pos],
                ' ':[no coverage pos],
                'all': [all pos]}
    """
    r_pos = 0
    q_pos = 0
    r_string = ""
    q_string = ""
    res = ""
    base_trans_counts = {'A':[],
                         'T':[],
                         'G':[],
                         'C':[],
                         'N':[],
                         ' ':[], 
                         'all':[], 
                         'CT':[], 
                         'GA':[]}
    for c in cigartuple:
        # [M, =, X] - alignment match (can be a sequence match or mismatch)
        if c[0] in [0, 7, 8]:
            r_string += reference[r_pos:r_pos+c[1]]
            q_string += query[q_pos:q_pos+c[1]]
            r_pos += c[1]
            q_pos += c[1]
        # I, S - Insertion of soft clipping
        if c[0] in [1, 4]:
            r_string += " "*c[1]
            q_string += query[q_pos:q_pos+c[1]]
            r_pos = r_pos
            q_pos += c[1]
        # D - Deletion in query: add space in query string
        if c[0] in [2, 3]:
            r_string += reference[r_pos:r_pos+c[1]]
            q_string += " "*c[1]
            r_pos += c[1]
            q_pos = q_pos

    for i in range(len(r_string)):
        r_char = r_string[i].upper()
        q_char = q_string[i].upper()
        base_trans_counts[q_char].append(i)
        if q_char in ['A','T','G','C']:
             base_trans_counts['all'].append(i)
        if r_char != q_char:
            if r_char == "C" and q_char == "T":
                base_trans_counts['CT'].append(i)
            if r_char == "G" and q_char == "A":
                base_trans_counts['GA'].append(i)

    if show_al:
        res += "R " + r_string + "\n  "
        for i in range(0, min(len(r_string), len(q_string))):
            if r_string[i] == q_string[i]:
                res += ("|")
            else:
                res += (" ")
        res += "\nQ " + q_string
        print(res)


    return(base_trans_counts)



from setuptools import setup, find_packages
import codecs
import os.path

def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()

def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")

setup(
    name='pydamage',
    version=get_version("pydamage/__init__.py"),
    description='Damage parameter estimation for ancient DNA',
    long_description=open("README.md").read(),
    url='https://github.com/maxibor/pydamage',
    long_description_content_type="text/markdown",
    license='GNU-GPLv3',
    python_requires=">=3.6",
    install_requires=[
        'click',
        'numpy',
        'pandas',
        'pysam',
        'scipy',
        'statsmodels',
        'matplotlib',
        'tqdm'
    ],
    packages=find_packages(include=['pydamage']),
    entry_points={
        'console_scripts': [
            'pydamage = pydamage.cli:cli'
        ]
    }
)
